import pytest

from gso.products.product_blocks.bgp_session import IPFamily
from gso.products.product_types.nren_l3_core_service import NRENL3CoreService, NRENL3CoreServiceType
from gso.utils.shared_enums import APType
from test.workflows import extract_state, run_workflow


@pytest.mark.parametrize("l3_core_service_type", [NRENL3CoreServiceType.GEANT_IP, NRENL3CoreServiceType.IAS])
@pytest.mark.workflow()
def test_modify_nren_l3_core_service_remove_edge_port_success(
    nren_l3_core_service_subscription_factory, l3_core_service_type
):
    subscription_id = nren_l3_core_service_subscription_factory(nren_l3_core_service_type=l3_core_service_type)
    subscription = NRENL3CoreService.from_subscription(subscription_id)
    access_port = subscription.nren_l3_core_service.nren_ap_list[0]
    input_form_data = [
        {"subscription_id": subscription_id},
        {
            "access_ports": [
                {
                    "edge_port": str(access_port.sbp.edge_port.owner_subscription_id),
                    "ap_type": APType.LOAD_BALANCED,
                }
            ]  # The factory generates a subscription with two Access Ports, this will remove the second one.
        },
        {},
    ]

    result, _, _ = run_workflow("modify_nren_l3_core_service", input_form_data)

    state = extract_state(result)
    subscription = NRENL3CoreService.from_subscription(state["subscription_id"])
    assert len(subscription.nren_l3_core_service.nren_ap_list) == 1
    assert subscription.nren_l3_core_service.nren_ap_list[0].ap_type == APType.LOAD_BALANCED


@pytest.mark.parametrize("l3_core_service_type", [NRENL3CoreServiceType.GEANT_IP, NRENL3CoreServiceType.IAS])
@pytest.mark.workflow()
def test_modify_nren_l3_core_service_add_new_edge_port_success(
    nren_l3_core_service_subscription_factory,
    edge_port_subscription_factory,
    partner_factory,
    faker,
    l3_core_service_type,
):
    partner = partner_factory()
    new_edge_port = edge_port_subscription_factory(partner=partner)
    subscription_id = nren_l3_core_service_subscription_factory(
        partner=partner, nren_l3_core_service_type=l3_core_service_type
    )
    subscription = NRENL3CoreService.from_subscription(subscription_id)
    input_form_data = [
        {"subscription_id": subscription_id},
        {
            "access_ports": [
                {
                    "edge_port": str(port.sbp.edge_port.owner_subscription_id),
                    "ap_type": port.ap_type,
                }
                for port in subscription.nren_l3_core_service.nren_ap_list
            ]
            + [
                {
                    "edge_port": str(new_edge_port),
                    "ap_type": APType.BACKUP,
                }
            ]
        },
        {},  # The existing SBPs are unchanged
        {},
        {  # Adding configuration for the new SBP
            "geant_sid": faker.geant_sid(),
            "vlan_id": faker.vlan_id(),
            "ipv4_address": faker.ipv4(),
            "ipv6_address": faker.ipv6(),
            "v4_bgp_peer": {
                "authentication_key": faker.password(),
                "peer_address": faker.ipv4(),
                "bfd_settings": {"bfd_enabled": False},
            },
            "v6_bgp_peer": {
                "authentication_key": faker.password(),
                "peer_address": faker.ipv6(),
                "bfd_settings": {"bfd_enabled": False},
            },
            "v4_bfd_settings": {"bfd_enabled": False},
            "v6_bfd_settings": {"bfd_enabled": False},
        },
    ]

    result, _, _ = run_workflow("modify_nren_l3_core_service", input_form_data)

    state = extract_state(result)
    subscription = NRENL3CoreService.from_subscription(state["subscription_id"])
    assert len(subscription.nren_l3_core_service.nren_ap_list) == 3


@pytest.fixture()
def sbp_input_form_data(faker):
    def _generate_form_data():
        return {
            "geant_sid": faker.geant_sid(),
            "is_tagged": True,
            "vlan_id": faker.vlan_id(),
            "ipv4_address": faker.ipv4(),
            "ipv6_address": faker.ipv6(),
            "custom_firewall_filters": True,
            "v4_bfd_settings": {
                "bfd_enabled": True,
                "bfd_interval": faker.pyint(),
                "bfd_multiplier": faker.pyint(),
            },
            "v6_bfd_settings": {
                "bfd_enabled": True,
                "bfd_interval": faker.pyint(),
                "bfd_multiplier": faker.pyint(),
            },
            "v4_bgp_peer": {
                "bfd_settings": {
                    "bfd_enabled": True,
                    "bfd_interval": faker.pyint(),
                    "bfd_multiplier": faker.pyint(),
                },
                "has_custom_policies": True,
                "authentication_key": faker.password(),
                "multipath_enabled": True,
                "send_default_route": True,
                "is_passive": True,
                "peer_address": faker.ipv4(),
                "add_v4_multicast": True,
            },
            "v6_bgp_peer": {
                "bfd_settings": {
                    "bfd_enabled": True,
                    "bfd_interval": faker.pyint(),
                    "bfd_multiplier": faker.pyint(),
                },
                "has_custom_policies": True,
                "authentication_key": faker.password(),
                "multipath_enabled": True,
                "send_default_route": True,
                "is_passive": True,
                "peer_address": faker.ipv6(),
                "add_v6_multicast": True,
            },
        }

    return _generate_form_data


@pytest.mark.parametrize("l3_core_service_type", [NRENL3CoreServiceType.GEANT_IP, NRENL3CoreServiceType.IAS])
@pytest.mark.workflow()
def test_modify_nren_l3_core_service_modify_edge_port_success(
    faker, nren_l3_core_service_subscription_factory, l3_core_service_type, sbp_input_form_data
):
    subscription_id = nren_l3_core_service_subscription_factory(nren_l3_core_service_type=l3_core_service_type)
    subscription = NRENL3CoreService.from_subscription(subscription_id)
    new_sbp_data = [sbp_input_form_data(), sbp_input_form_data()]
    input_form_data = [
        {"subscription_id": subscription_id},
        {
            "access_ports": [
                {
                    "edge_port": str(port.sbp.edge_port.owner_subscription_id),
                    "ap_type": port.ap_type,
                }
                for port in subscription.nren_l3_core_service.nren_ap_list
            ]
        },
        {**new_sbp_data[0]},
        {**new_sbp_data[1]},
    ]

    result, _, _ = run_workflow("modify_nren_l3_core_service", input_form_data)

    state = extract_state(result)
    subscription = NRENL3CoreService.from_subscription(state["subscription_id"])
    assert len(subscription.nren_l3_core_service.nren_ap_list) == 2

    for i in range(2):
        assert subscription.nren_l3_core_service.nren_ap_list[i].sbp.geant_sid == new_sbp_data[i]["geant_sid"]
        assert subscription.nren_l3_core_service.nren_ap_list[i].sbp.is_tagged == new_sbp_data[i]["is_tagged"]
        assert subscription.nren_l3_core_service.nren_ap_list[i].sbp.vlan_id == new_sbp_data[i]["vlan_id"]
        assert (
            str(subscription.nren_l3_core_service.nren_ap_list[i].sbp.ipv4_address) == new_sbp_data[i]["ipv4_address"]
        )
        assert (
            str(subscription.nren_l3_core_service.nren_ap_list[i].sbp.ipv6_address) == new_sbp_data[i]["ipv6_address"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.custom_firewall_filters
            == new_sbp_data[i]["custom_firewall_filters"]
        )

        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].bfd_settings.bfd_enabled
            == new_sbp_data[i]["v4_bgp_peer"]["bfd_settings"]["bfd_enabled"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].bfd_settings.bfd_interval
            == new_sbp_data[i]["v4_bgp_peer"]["bfd_settings"]["bfd_interval"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].bfd_settings.bfd_multiplier
            == new_sbp_data[i]["v4_bgp_peer"]["bfd_settings"]["bfd_multiplier"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].has_custom_policies
            == new_sbp_data[i]["v4_bgp_peer"]["has_custom_policies"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].authentication_key
            == new_sbp_data[i]["v4_bgp_peer"]["authentication_key"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].multipath_enabled
            == new_sbp_data[i]["v4_bgp_peer"]["multipath_enabled"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].send_default_route
            == new_sbp_data[i]["v4_bgp_peer"]["send_default_route"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].is_passive
            == new_sbp_data[i]["v4_bgp_peer"]["is_passive"]
        )
        assert (
            str(subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].peer_address)
            == new_sbp_data[i]["v4_bgp_peer"]["peer_address"]
        )
        assert (
            bool(
                IPFamily.V4MULTICAST
                in subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[0].families
            )
            == new_sbp_data[i]["v4_bgp_peer"]["add_v4_multicast"]
        )

        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].bfd_settings.bfd_enabled
            == new_sbp_data[i]["v6_bgp_peer"]["bfd_settings"]["bfd_enabled"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].bfd_settings.bfd_interval
            == new_sbp_data[i]["v6_bgp_peer"]["bfd_settings"]["bfd_interval"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].bfd_settings.bfd_multiplier
            == new_sbp_data[i]["v6_bgp_peer"]["bfd_settings"]["bfd_multiplier"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].has_custom_policies
            == new_sbp_data[i]["v6_bgp_peer"]["has_custom_policies"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].authentication_key
            == new_sbp_data[i]["v6_bgp_peer"]["authentication_key"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].multipath_enabled
            == new_sbp_data[i]["v6_bgp_peer"]["multipath_enabled"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].send_default_route
            == new_sbp_data[i]["v6_bgp_peer"]["send_default_route"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].is_passive
            == new_sbp_data[i]["v6_bgp_peer"]["is_passive"]
        )
        assert (
            str(subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].peer_address)
            == new_sbp_data[i]["v6_bgp_peer"]["peer_address"]
        )
        assert (
            bool(
                IPFamily.V6MULTICAST
                in subscription.nren_l3_core_service.nren_ap_list[i].sbp.bgp_session_list[1].families
            )
            == new_sbp_data[i]["v6_bgp_peer"]["add_v6_multicast"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.v4_bfd_settings.bfd_enabled
            == new_sbp_data[i]["v4_bfd_settings"]["bfd_enabled"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.v4_bfd_settings.bfd_interval
            == new_sbp_data[i]["v4_bfd_settings"]["bfd_interval"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.v4_bfd_settings.bfd_multiplier
            == new_sbp_data[i]["v4_bfd_settings"]["bfd_multiplier"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.v6_bfd_settings.bfd_enabled
            == new_sbp_data[i]["v6_bfd_settings"]["bfd_enabled"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.v6_bfd_settings.bfd_interval
            == new_sbp_data[i]["v6_bfd_settings"]["bfd_interval"]
        )
        assert (
            subscription.nren_l3_core_service.nren_ap_list[i].sbp.v6_bfd_settings.bfd_multiplier
            == new_sbp_data[i]["v6_bfd_settings"]["bfd_multiplier"]
        )
